#include "nodes/base/base.hpp"
#include "nodes/stream/streamNode.hpp"

namespace Node
{

void StreamNode::work()
{
    cv::VideoCapture cap(stream_url_);
    if (!cap.isOpened())
    {
        std::cerr << "Error: cannot open camera" << std::endl;
        return;
    }
    printf("StreamNode %s\n", name_.c_str());
    printf("stram url: %s\n", stream_url_.c_str());
    printf("fps: %f\n", cap.get(cv::CAP_PROP_FPS));
    printf("width: %f\n", cap.get(cv::CAP_PROP_FRAME_WIDTH));
    printf("height: %f\n", cap.get(cv::CAP_PROP_FRAME_HEIGHT));

    while (running_)
    {
        cv::Mat frame;
        cap >> frame;
        frame_count_++;
        if (frame_count_ % skip_frame_ != 0)
        {
            // printf("Skip frame %d\n", frame_count_);
            continue;
        }
        if (frame.empty())
        {
            std::cerr << "Error: cannot read frame" << std::endl;
            break;
        }

        auto metaData = std::make_shared<meta::MetaData>();
        metaData->image = frame;
        metaData->from = name_;

        for (auto& output_buffer : output_buffers_)
        {
            output_buffer.second->push(metaData);
            // printf("Node %s push data to %s\n", name_.c_str(), output_buffer.first.c_str());
        }
    }
}


}   // namespace Node